export interface Cache {
    _keys?: any[];
    _values?: any[];
    has: (value: any) => boolean;
    set: (key: any, value: any) => void;
    get: (key: any) => any;
}
export declare type InternalCopier = <Value = any>(value: Value, cache: Cache) => Value;
export declare type ObjectCloner = <Value>(object: Value, realm: Realm, handleCopy: InternalCopier, cache: Cache) => Value;
export declare type Realm = Record<string, any>;
declare function createCacheModern(): Cache;
/**
 * Get a new cache object to prevent circular references.
 */
export declare const createCache: typeof createCacheModern;
/**
 * Get an empty version of the object with the same prototype it has.
 */
export declare function getCleanClone(object: any, realm: Realm): any;
/**
 * Get a copy of the object based on loose rules, meaning all enumerable keys
 * and symbols are copied, but property descriptors are not considered.
 */
export declare function getObjectCloneLoose(object: any, realm: Realm, handleCopy: InternalCopier, cache: Cache): any;
/**
 * Get a copy of the object based on strict rules, meaning all keys and symbols
 * are copied based on the original property descriptors.
 */
export declare function getObjectCloneStrict(object: any, realm: Realm, handleCopy: InternalCopier, cache: Cache): any;
declare function getRegExpFlagsModern(regExp: RegExp): string;
/**
 * Get the flags to apply to the copied regexp.
 */
export declare const getRegExpFlags: typeof getRegExpFlagsModern;
export {};
