import type { Realm } from './utils';
export interface Options {
    isStrict?: boolean;
    realm?: Realm;
}
export interface StrictOptions extends Omit<Options, 'isStrict'> {
}
/**
 * Copy an value deeply as much as possible.
 *
 * If `strict` is applied, then all properties (including non-enumerable ones)
 * are copied with their original property descriptors on both objects and arrays.
 *
 * The value is compared to the global constructors in the `realm` provided,
 * and the native constructor is always used to ensure that extensions of native
 * objects (allows in ES2015+) are maintained.
 */
export declare function copy<Value>(value: Value, options?: Options): Value;
/**
 * Copy the value with `strict` option pre-applied.
 */
export declare function copyStrict(value: any, options?: StrictOptions): any;
